"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const typescriptVfs_js_1 = require("../../../utils/typescriptVfs.js");
function main(options) {
    const { filename } = options;
    const { getCompletions, getQuickInfo } = (0, typescriptVfs_js_1.configVfs)({
        filename,
        initialCode: `\
const inflection: GraphileBuild.Inflection = null as any;
inflection`,
    });
    let outputText = ``;
    let last = "";
    let inCode = false;
    function out(text = "") {
        if (text === "" && last === "" && !inCode) {
            return;
        }
        let exitCode = false;
        if (!inCode && text.startsWith("```")) {
            inCode = true;
        }
        else if (text.startsWith("```")) {
            exitCode = true;
        }
        outputText += (inCode ? chalk_1.default.green(text) : text) + "\n";
        last = text;
        if (exitCode) {
            inCode = false;
        }
    }
    out(chalk_1.default.whiteBright.bold(`# Custom \`GraphileBuild.Inflection\` Reference`));
    out();
    out(`\
This reference was autogenerated by \`graphile inflection list\` using your
local configuration file to determine the inflectors that are available based
on the plugins and presets you use. You should regenerate it from time to time
(for example, when you upgrade a module, or add/remove modules).`);
    out();
    const completions = [...(getCompletions(".")?.entries ?? [])];
    completions.sort((a, z) => a.name.localeCompare(z.name, "en-US"));
    let later = [];
    function outLater(str) {
        later.push(str);
    }
    const entries = [];
    const INDENT = 2;
    for (const entry of completions) {
        const key = entry.name;
        const withProperty = (0, typescriptVfs_js_1.accessKey)(key);
        const info = getQuickInfo(withProperty);
        //entries.push(
        //  `${chalk.cyanBright(key)}${prettyQuickInfoDisplayParts(info)};`,
        //);
        const doc = (0, typescriptVfs_js_1.tightDocumentation)(info, 80 - INDENT - 7);
        entries.push(`${doc
            ? chalk_1.default.gray(`/** ${doc} */
`)
            : ``}${chalk_1.default.cyanBright(key)}${(0, typescriptVfs_js_1.tightDisplayParts)(info, 80 - INDENT - key.length)}`);
        outLater(chalk_1.default.whiteBright.bold(`## ${chalk_1.default.cyanBright.bold(key)}`));
        outLater();
        outLater((0, typescriptVfs_js_1.prettyDocumentation)(info?.documentation));
        outLater();
        outLater("```ts");
        outLater(key + (0, typescriptVfs_js_1.prettyQuickInfoDisplayParts)(info));
        outLater("```");
        outLater();
    }
    if (entries.length) {
        out("```ts");
        out(chalk_1.default.cyanBright(`interface GraphileBuild.Inflection`) + ` {`);
        for (const entry of entries) {
            out("  " + entry.replace(/\n/g, "\n  "));
        }
        out("}");
        out("```");
        out();
    }
    for (const line of later) {
        out(line);
    }
    later = [];
    return outputText.trim() + "\n";
}
exports.main = main;
//# sourceMappingURL=main.js.map